/**
 * deconvolve_dft.h   This is a demo library for unfolding using dft.
 */


#ifndef __DECONVOLVE_DFT
#define __DECONVOLVE_DFT


/* Types. */
#include <unfold_iterative.h>


/**
 * Procedurefor deconvolution using dft one dimensional 
 * probability space. The parameters and return value of this function 
 * is the following.
 * 
 * @param  nbinsin	Total number of bins in the input histogram.
 * @param  inputpdf	Input probability density function values at nbins 
 *         		space point locations.
 * @param  foldingop	The folding operator.
 * @param  parameters	Additional parameters passed to the folding operator.
 * @param  nbinsout	Total number of bins in the output histogram.
 * @param  unfoldedpdf	Output probability density function values at the 
 *         		same locations as inputpdf.
 * @return errorcode	Nonzero on error (e.g. if nbinsout!=nbinsin).
 */
EXTERN int deconvolve_dft(const int nbinsin, const double *inputpdf, 
			  const folding foldingop, const void *parameters, 
			  const int nbinsout, double *unfoldedpdf);


#endif /* __DECONVOLVE_DFT */
