/**
 * Matrix inversion and linear equation solving with Gauss elimination.
 */


#ifndef __INVERT
#define __INVERT


/** Provide extern "C" for C++ compilers instead of extern. */
#ifdef __cplusplus
#define EXTERN extern "C"
#else
#define EXTERN extern
#endif


/** Square matrix structure. */
typedef struct
{
  int dim;
  double **d;
}
matrix_t;


/** Vector structure. */
typedef struct
{
  int dim;
  double *d;
}
vector_t;


/** Allocate, deallocate vector and square matrix. */
EXTERN double* allocate_vector(const int dim);
EXTERN double** allocate_squarematrix(const int dim);
EXTERN void free_vector(double *v);
EXTERN void free_squarematrix(double **m, const int dim);


/** Solve linear equation with Gauss elimination. */
EXTERN int solve(const matrix_t* min, const vector_t* vin, vector_t* vout);


/** Invert matrix by solving linear equations. */
EXTERN int invert(const matrix_t* min, matrix_t* mout);


#endif /* __INVERT*/
