/**
 * rnd.h  Random number generation tools. Based on drand48() and 
 *        srand48() functions.
 */


#ifndef __RND_H
#define __RND_H


/** Provide extern "C" for C++ compilers instead of extern. */
#ifdef __cplusplus
#define EXTERN extern "C"
#else
#define EXTERN extern
#endif


/** Initialize random seed by hand. */
EXTERN void init_rnd(const unsigned int seed);

/** This function returns random seed. */
EXTERN unsigned int get_randomseed();

/** Generate random number with uniform distribution (on interval [a,b[). */
EXTERN double uniform(const double a, const double b);

/** Generate random number with triangle distribution (on interval ]a-w,a+w[). */
EXTERN double triang(const double a, const double w);

/** Generate random number with Gauss distribution (mean value a, spread s). */
EXTERN double gauss(const double a, const double s);

/** Generate random number with x |-> 1/(pi*G)*1/(1+((x-a)*(x-a))/(G*G)), i.e. 
    Cauchy distribution, with center a and full half width G. */
EXTERN double cauchy(const double a, const double G);

/** Generate random number with exponential distribution with decay parameter 
    d, on the interval [0,infinity[. */
EXTERN double exponential(const double d);

/** Generate random number with x |-> 1/d^2*x*exp(-x/d) density function, 
    on the interval [0,infinity[. */
EXTERN double idexponential(const double d);

/** Generate random number with E |-> E*(n-1)*(n-2)/(n*T)^2*(1+E/(n*T))^(-n)
    density function, on the interval [0,infinity[. */
EXTERN double idtsallis(const double T, const double n);


#endif /* __RND_H */
