/**
 * unfold_inverse.h   This is a demo library for unfolding using 
 *                    matrix inversion.
 */


#ifndef __UNFOLD_INVERSE
#define __UNFOLD_INVERSE


/* Types. */
#include <unfold_iterative.h>


/**
 * Procedurefor unfolding using matrix inversion in any dimensional 
 * probability space. The parameters and return value of this function 
 * is the following.
 * 
 * @param  nbinsin	Total number of bins in the input histogram.
 * @param  inputpdf	Input probability density function values at nbins 
 *         		space point locations.
 * @param  foldingOp	The folding operator.
 * @param  parameters	Additional parameters passed to the folding operator.
 * @param  nbinsout	Total number of bins in the output histogram.
 * @param  unfoldedpdf	Output probability density function values at the 
 *         		same locations as inputpdf.
 * @return errorcode	Nonzero on error (e.g. if nbinsout!=nbinsin).
 */
EXTERN int unfold_inverse(const int nbinsin, const double *inputpdf, 
			  const folding foldingOp, const void *parameters, 
			  const int nbinsout, double *unfoldedpdf);


/**
 * Procedureto determine matrix norm of an unfolding operator in a given 
 * discretization in any dimensional probability space. The parameters and 
 * return value of this function is the following.
 * 
 * @param  nbinsin	Total number of bins in the input histogram.
 * @param  foldingOp	The folding operator.
 * @param  parameters	Additional parameters passed to the folding operator.
 * @param  nbinsout	Total number of bins in the output histogram.
 * @param  opNorm	Pointer to return the computed operator norm.
 * @return errorcode	Nonzero on error (e.g. if nbinsout!=nbinsin).
 */
EXTERN int unfold_inverseoperatornorm(const int nbinsin, 
				      const folding foldingOp, 
				      const void *parameters, 
				      const int nbinsout, 
				      double *opNorm);


#endif /* __UNFOLD_INVERSE */
