/**
 * plot_demo.C  This is a BLOP-plot script for plotting results.
 */


#ifndef __CINT__
#include <blop.h>
using namespace std;
using namespace blop;
#endif


int main(int argc, char* argv[])
{

    // Plot results for unfolding by deconvolution with DFT.
    set::clear();
    set::title("Unfolding by deconvolution with DFT demo");
    set::xlabel("Probability variable");
    set::ylabel("Probability density");
    set::xrange(unset, unset);
    mplot("out/data/unfoldedpdf_dft.dat", _1, _2).lw(3.0*LW).ds(lines()).ac(red).legend("Input Cauchy pdf");
    mplot("out/data/unfoldedpdf_dft.dat", _1, _4, _5).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(fsquare()).ac(green).legend("Measured pdf");
    mplot("out/data/unfoldedpdf_dft.dat", _1, _6, _7).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(ftriangle()).ac(blue).legend("Unfolded pdf");
    blopeps::print("out/bplots/unfoldedpdf_dft.beps");

    // Plot results for unfolding by matrix inversion.
    set::clear();
    set::title("Unfolding by matrix inversion demo");
    set::xlabel("Probability variable");
    set::ylabel("Probability density");
    set::xrange(unset, unset);
    mplot("out/data/unfoldedpdf_inverse.dat", _1, _2).lw(3.0*LW).ds(lines()).ac(red).legend("Input Cauchy pdf");
    mplot("out/data/unfoldedpdf_inverse.dat", _1, _4, _5).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(fsquare()).ac(green).legend("Measured pdf");
    mplot("out/data/unfoldedpdf_inverse.dat", _1, _6, _7).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(ftriangle()).ac(blue).legend("Unfolded pdf");
    blopeps::print("out/bplots/unfoldedpdf_inverse.beps");

    // Plot results for unfolding by Bayesian method.
    int nIterations_bayesian=0;
    double errorContent_bayesian=0.0;
    int entries_bayesian=0;
    ifstream ifile_bayesian;
    istringstream iss_bayesian;
    string line_bayesian, word_bayesian;
    ifile_bayesian.open("out/data/unfoldedpdf_bayesian_report.dat");
    while ( getline(ifile_bayesian, line_bayesian) )
    {
	if ( line_bayesian.length()==0 ) continue;
	if ( line_bayesian[0]=='#' ) continue;
	iss_bayesian.str(line_bayesian);
	if ( !(iss_bayesian>>word_bayesian) ) { cerr<<"[plot] Could not read word.\n"; exit(1); }
	if ( word_bayesian=="nIterations=" ) iss_bayesian>>nIterations_bayesian;
	if ( word_bayesian=="errorContent=" ) iss_bayesian>>errorContent_bayesian;
	if ( word_bayesian=="entries=" ) iss_bayesian>>entries_bayesian;
	iss_bayesian.str("");
	iss_bayesian.clear();
    }
    ifile_bayesian.close();
    ifile_bayesian.clear();
    errorContent_bayesian*=100.0;
    set::clear();
    set::title(var("Bayesian unfolding demo (")&nIterations_bayesian&" iterations, error content="&errorContent_bayesian&"\\\%)");
    set::xlabel("Probability variable");
    set::ylabel("Probability density");
    set::xrange(unset, unset);
    mplot("out/data/unfoldedpdf_bayesian.dat", _1, _2).lw(3.0*LW).ds(lines()).ac(red).legend("Input Cauchy pdf");
    mplot("out/data/unfoldedpdf_bayesian.dat", _1, _4, _5).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(fsquare()).ac(green).legend("Measured pdf");
    mplot("out/data/unfoldedpdf_bayesian.dat", _1, _6, _7).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(ftriangle()).ac(blue).legend("Unfolded pdf");
    label::fdraw(var("Statistics = ")&var(entries_bayesian)&" entries", 0.0+2.0*width("0"), 1.0-2.0*height("0")).align(sym::left, sym::top).textcolor(gray);
    blopeps::print("out/bplots/unfoldedpdf_bayesian.beps");

    // Plot d'Alembert convergence factors.
    set::clear();
    set::title("d'Alembert convergence test of Bayesian method");
    set::xlabel("Iteration order");
    set::ylabel("d'Alembert convergence factor");
    set::xrange(0.0, unset);
    set::yrange(0.0, 2.0);
    mplot("out/data/unfoldedpdf_bayesian_dalembert.dat", _1, _2).lw(3.0*LW).ds(points()).pt(fcircle()).ac(magenta).legend("");
    mplot("out/data/unfoldedpdf_bayesian_dalembert.dat", _1, _2).lw(3.0*LW).ds(lines()).ac(magenta).legend("");
    line::fdraw(0.0, y1len(1.0))(1.0, y1len(1.0)).linestyle(sym::dotted);
    blopeps::print("out/bplots/unfoldedpdf_bayesian_dalembert.beps");

    // Plot results for unfolding by iterative unfolding.
    int nIterations=0;
    double errorContent=0.0;
    int entries=0;
    ifstream ifile;
    istringstream iss;
    string line, word;
    ifile.open("out/data/unfoldedpdf_iterative_report.dat");
    while ( getline(ifile, line) )
    {
	if ( line.length()==0 ) continue;
	if ( line[0]=='#' ) continue;
	iss.str(line);
	if ( !(iss>>word) ) { cerr<<"[plot] Could not read word.\n"; exit(1); }
	if ( word=="nIterations=" ) iss>>nIterations;
	if ( word=="errorContent=" ) iss>>errorContent;
	if ( word=="entries=" ) iss>>entries;
	iss.str("");
	iss.clear();
    }
    ifile.close();
    ifile.clear();
    errorContent*=100.0;
    set::clear();
    set::title(var("Iterative unfolding demo (")&nIterations&" iterations, error content="&errorContent&"\\\%)");
    set::xlabel("Probability variable");
    set::ylabel("Probability density");
    set::xrange(unset, unset);
    mplot("out/data/unfoldedpdf_iterative.dat", _1, _2).lw(3.0*LW).ds(lines()).ac(red).legend("Input Cauchy pdf");
    mplot("out/data/unfoldedpdf_iterative.dat", _1, _4, _5).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(fsquare()).ac(green).legend("Measured pdf");
    mplot("out/data/unfoldedpdf_iterative.dat", _1, _6, _7).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(ftriangle()).ac(blue).legend("Unfolded pdf using");
    label::fdraw(var("Statistics = ")&var(entries)&" entries", 0.0+2.0*width("0"), 1.0-2.0*height("0")).align(sym::left, sym::top).textcolor(gray);
    blopeps::print("out/bplots/unfoldedpdf_iterative.beps");

    // Plot d'Alembert convergence factors.
    set::clear();
    set::title("d'Alembert convergence test of iterative unfolding");
    set::xlabel("Iteration order");
    set::ylabel("d'Alembert convergence factor");
    set::xrange(0.0, unset);
    set::yrange(0.0, 2.0);
    mplot("out/data/unfoldedpdf_iterative_dalembert.dat", _1, _2).lw(3.0*LW).ds(points()).pt(fcircle()).ac(magenta).legend("");
    mplot("out/data/unfoldedpdf_iterative_dalembert.dat", _1, _2).lw(3.0*LW).ds(lines()).ac(magenta).legend("");
    line::fdraw(0.0, y1len(1.0))(1.0, y1len(1.0)).linestyle(sym::dotted);
    blopeps::print("out/bplots/unfoldedpdf_iterative_dalembert.beps");

    return 0;
}
