/**
 * plot_example.C  This is a BLOP-plot script for plotting results.
 */


#ifndef __CINT__
#include <blop.h>
using namespace std;
using namespace blop;
#endif


// Custom colors.
color lightgreen(0.5, 1.0, 0.5);
color lightblue1(0.75, 0.75, 1.0);
color lightblue2(0.4, 0.4, 1.0);
color gray1(0.25, 0.25, 0.25);


// Intensity color map based on green.
double transform(const double val)
{
    return ::pow(val, 0.6);
}
color grayscale(double val, double min, double max)
{
    const double c=1.0;
    const double myval=transform(val);
    const double mymin=transform(min);
    const double mymax=transform(max);
    if ( myval<mymin ) return color(0.0, 0.0, 1.0);
    else if ( mymax<myval ) return color(1.0, 0.0, 0.0);
    double v=(myval-mymin)/(mymax-mymin);
    return color((1.0-c*v), (1.0-c*v), (1.0-c*v));
}


int main(int argc, char* argv[])
{
    // Input and folding pdfs.
    const double T=0.145;
    const double n=6.6;
    function eTpdf=_1*(n-1.0)*(n-2.0)/((n*T)*(n*T))*pow(1.0+_1/(n*T), -n)*charfunc(0.0, 10.0, true, true)(_1);
    const double a=0.150;
    const double b=0.847*0.847;
    const double c=0.074;
    function sigma=a+sqrt(b*_1)+c*_1;
    function fResponse=(_1>0.0)*1.0/sqrt(2.0*M_PI*sigma(_1)*sigma(_1))*exp(-(_2-_1)*(_2-_1)/(2.0*sigma(_1)*sigma(_1)));
    const double syst=0.05;
    const double maxIneff=syst;
    const double transitionE=1.0;
    const double scaleE=1.0;
    function fSyst=0.5*abs(1.0+tanh((_1-transitionE)/scaleE))*maxIneff;

    // Filename for pdfs.
    const string filenamereport="out/data/cmshcal_report.dat";
    const string filenamehist="out/data/cmshcal_iterative.dat";
    const string filenamedalembert="out/data/cmshcal_dalembert.dat";
    const string filenameerrorcontent="out/data/cmshcal_errorcontent.dat";

    // Read in some parameters from report file.
    int nIterations=0;
    double maxApproxErr=0.0;
    double maxStatErr=0.0;
    double maxSystErr=0.0;
    double entries=0;
    ifstream ifile;
    string line;
    istringstream iss;
    string word;
    ifile.open(filenamereport.c_str());
    while ( getline(ifile, line) )
    {
	if ( line.length()==0 ) continue;
	if ( line[0]=='#' ) continue;
	iss.str(line);
	if ( !(iss>>word) ) { cerr<<"[plot] Could not read word.\n"; exit(1); }
	if ( word=="nIterations=" ) iss>>nIterations;
	if ( word=="maxApproxErr=" ) iss>>maxApproxErr;
	if ( word=="maxStatErr=" ) iss>>maxStatErr;
	if ( word=="maxSystErr=" ) iss>>maxSystErr;
	if ( word=="entries=" ) iss>>entries;
	iss.str("");
	iss.clear();
    }
    ifile.close();
    ifile.clear();
    double errorContent=::sqrt(maxApproxErr*maxApproxErr+maxStatErr*maxStatErr+maxSystErr*maxSystErr);
    errorContent*=10000.0;
    errorContent=(int)errorContent/100.0;

    // Some dummy dgraphs for proper plotting.
    dgraph gUnfolded;
    gUnfolded.read(filenamehist);
    dgraph empty1;
    empty1.add(gUnfolded[0][0].dbl(), 0.0, 0.0);
    empty1.add(gUnfolded[1][0].dbl(), 0.0, 0.0);
    const double minE=gUnfolded[0][0].dbl();
    const double maxE=gUnfolded[gUnfolded.size()-1][0].dbl();

    // Plot results for unfolding by iterative unfolding for CMS-HCAL.
    set::clear();
    set::title(var("Linear iterative unfolding (")&nIterations&" iterations, combined error content="&errorContent&"\\\%)");
    set::xlabel("Energy [GeV]");
    set::ylabel("Probability density [1/GeV]");
    set::xrange(-2.0, unset);
    line::fdraw(0.0, y1len(0.0))(1.0, y1len(0.0)).linestyle(sym::dotted);
    label::fdraw(var("Statistics = ")&var(entries)&" entries", 1.0-4.0*width("0"), 0.5-2.0*height("0")).align(sym::right, sym::bottom).textcolor(gray1);
    mplot(empty1).lw(3.0*LW).ds(lines()).ac(red).legend("Input pdf (MC truth)");
    mplot(filenamehist, _1, _4-_6, _4+_6).ds(ybands()).ac(lightgreen).legend("");
    mplot(filenamehist, _1, _7-_10, _7+_10).ds(ybands()).ac(lightblue1).legend("");
    mplot(filenamehist, _1, _7-_9, _7+_9).ds(ybands()).ac(lightblue2).legend("");
    mplot(_1, eTpdf(_1)).nsamples(500).lw(3.0*LW).ds(lines()).ac(red).legend("");
    mplot(filenamehist, _1, _4, _5).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(fsquare()).ac(green).legend("Measured folded pdf (with stat.\\ and syst.\\ errors)");
    mplot(filenamehist, _1, _7, _8).lw(3.0*LW).ds(syerrorbars().clip_errorbars(true)).pt(ftriangle()).ac(blue).legend("Unfolded pdf (linear iterative unfolding)");
    blopeps::print("out/bplots/cmshcal_iterative_lin.beps");

    // Plot d'Alembert convergence factors for CMS-HCAL.
    set::clear();
    set::title("d'Alembert convergence test of iterative unfolding");
    set::xlabel("Iteration order");
    set::ylabel("d'Alembert convergence factor");
    set::xrange(0.0, unset);
    set::yrange(0.0, 2.0);
    mplot(filenamedalembert, _1, _2).lw(3.0*LW).ds(points()).pt(fcircle()).ac(magenta).legend("$L^{1}$");
    mplot(filenamedalembert, _1, _2).lw(3.0*LW).ds(lines()).ac(magenta).legend("");
    mplot(filenamedalembert, _1, _3).lw(3.0*LW).ds(points()).pt(fdiamond()).ac(cyan).legend("$L^{2}$");
    mplot(filenamedalembert, _1, _3).lw(3.0*LW).ds(lines()).ac(cyan).legend("");
    mplot(filenamedalembert, _1, _4).lw(3.0*LW).ds(points()).pt(ftriangledown()).ac(sienna).legend("$L^{\\infty}$");
    mplot(filenamedalembert, _1, _4).lw(3.0*LW).ds(lines()).ac(sienna).legend("");
    line::fdraw(0.0, y1len(1.0))(1.0, y1len(1.0)).linestyle(sym::dotted);
    blopeps::print("out/bplots/cmshcal_dalembert.beps");

    // Maximum bin probability errors for CMS-HCAL.
    set::clear();
    set::title("Evolution of approximation, statistical and systematic bin probability errors");
    set::xlabel("Iteration order");
    set::ylabel("Largest error of bin probability");
    frame::current().legendpos(sym::left, sym::top);
    set::xrange(0.0, unset);
    set::yrange(0.0, unset);
    mplot(filenameerrorcontent, _1, _2).lw(3.0*LW).ds(points()).pt(fcircle()).ac(magenta).legend("Largest approximation error");
    mplot(filenameerrorcontent, _1, _3).lw(3.0*LW).ds(points()).pt(fdiamond()).ac(cyan).legend("Largest statistical error");
    mplot(filenameerrorcontent, _1, _4).lw(3.0*LW).ds(points()).pt(ftriangledown()).ac(sienna).legend("Largest systematic error");
    blopeps::print("out/bplots/cmshcal_errorcontent.beps");

    // Plot response function.
    set::clear();
    set::title("Response function of the test system");
    set::xlabel("Input energy [GeV]");
    set::ylabel("Instrumentally measured energy [GeV]");
    set::xrange(0.0, maxE);
    set::yrange(-2.0, maxE);
    mplot(_1, _2, fResponse(_1, _2)).nsamples(50).ds(cboxes(grayscale).title("Probability density")).legend("");
    blopeps::print("out/bplots/cmshcal_responsefunction.beps");

    // Plot uncorrected inefficiency.
    set::clear();
    set::title("Uncorrected systematic distortion of the system");
    set::xlabel("Instrumentally measured energy [GeV]");
    set::ylabel("Applied systematic distortion on folded pdf [\\%]");
    set::xrange(-2.0, maxE);
    set::yrange(0.0, 10.0);
    mplot(_1, fSyst(_1)*100.0).lw(3.0*LW).nsamples(500).ds(lines()).ac(gray1).legend("");
    blopeps::print("out/bplots/cmshcal_systdistortion.beps");

    return 0;
}
